#!/bin/ksh

echo \*\*\*
echo \*\*\*
echo \*\*\* This script makes the following assumptions:
echo \*\*\* 1. The script is being executed on the Business Intelligence middle tier server.
echo \*\*\* "2. ORACLE_HOME environment variable is set to the BI Home (Not DB Home)"
echo \*\*\* 3. The bi_user has not been previously installed.
echo \*\*\* 4. Oracle Command Line and Discoverer For OLAP has been installed.
echo \*\*\*

#Check for the number of arguments.
if [ $# -ne 7 ]
then	
	echo "Usage: $0 hostname port sid tnsnames_entry system_password D4OSYS_password location_script"
	echo   "hostname             The database host name."
	echo   "port                 The database port."
	echo   "sid                  The database sid."
	echo   "tnsnames_entry       The tnsnames entry for the database server."
	echo   "system_password      The SYSTEM password."
	echo   "D4OSYS_password      The D4OSYS password."
	echo   "location_script      Fully qualified location of the script."
	echo   ""
	echo   "For example:"
	echo   "$0 testmachine 1521 10g tnsentry manager d4osys /tmp/scripts/samples/discoverer"
	exit 1
fi

#Set Common Vars
HOSTNAME=$1
PORT=$2
SID=$3
TNSNAMES=$4
SYSTEM_PASSWORD=$5
D4OSYS_PASSWORD=$6
SCRIPT_LOCATION=$7
BI_HOME=$ORACLE_HOME

# Display the arguments to the user to verify they are correct 
echo "You have specified the following arguments to install_olap.sh"

echo "hostname		$HOSTNAME"
echo "port		$PORT"
echo "sid		$SID"
echo "tnsnames_entry	$TNSNAMES"
echo "system_password 	$SYSTEM_PASSWORD"
echo "D4OSYS_password 	$D4OSYS_PASSWORD"
echo "location_script		$SCRIPT_LOCATION"
echo 

#Check ORACLE_HOME has been set

echo Validating ORACLE_HOME is set.

txt=`env | grep "^ORACLE_HOME="`
if [ "$txt" = "" ]
then
	echo "Env variable ORACLE_HOME not set"
	echo "The ORACLE_HOME must be set to the BI Home and not the DB Home."
	exit 1
fi

#Determine whether the OLAP option is installed
echo "Checking if OLAP option is installed"
$ORACLE_HOME/bin/sqlplus -S system/$SYSTEM_PASSWORD@$TNSNAMES @check_olap.sql

grep "OLAP Analytic Workspace" check_olap.log > /dev/null
if [ $? -ne 0 ]
then
	echo \*\*\*
	echo \*\*\* WARNING: OLAP Analytic Workspace component not installed.
	echo \*\*\*
	OLAP_INSTALLED="FALSE"
fi
grep "Oracle OLAP API" check_olap.log > /dev/null
if [ $? -ne 0 ]
then
	echo \*\*\*
	echo \*\*\* WARNING: OLAP Analytic API not installed.
	echo \*\*\*
	OLAP_INSTALLED="FALSE"
fi
grep "OLAP Catalog" check_olap.log > /dev/null
if [ $? -ne 0 ]
then
	echo \*\*\*
	echo \*\*\* WARNING: OLAP Analytic Catalog component not installed.
	echo \*\*\*
	OLAP_INSTALLED="FALSE"
fi

echo OLAP Components are present in the target database.
OLAP_INSTALLED="TRUE"

#Check the SYSTEM password ...
echo Validating SYSTEM password ...
$ORACLE_HOME/bin/sqlplus -S system/$SYSTEM_PASSWORD@$TNSNAMES @logon.sql
if [ $? -ne 0 ]
then
        echo \*\*\*
        echo \*\*\* Error: Cannot authenticate using supplied SYSTEM credentials.
        echo \*\*\* Please validate the supplied password and tnsnames, and try again.
        echo \*\*\*
        exit 1
fi

#Check the location of the BI_HOME
echo "Validating the location of BI_HOME exists ..."
if [ ! -a $BI_HOME/bin ]
then
	echo \*\*\*
	echo \*\*\* Error: Invalid location for BI Home.
	echo \*\*\* Please ensure the location of the BI Home is valid, and try again.
	echo \*\*\*
	exit 1
fi

#Check for D4O
if [ $OLAP_INSTALLED = "TRUE" ] 
then
	echo "Validating D4O installed - \$BI_HOME/sysman/webapps/emd/WEB-INF/lib/d4o.jar ..."
	if [ ! -a $BI_HOME/sysman/webapps/emd/WEB-INF/lib/d4o.jar ]
	then
		echo \*\*\*
		echo \*\*\* Error: Cannot find D4O - $BI_HOME/discoverer/lib/d4o.jar.
		echo \*\*\* Please ensure D4O is installed, and try again.
		echo \*\*\*
		exit 1
	fi
fi

#Check the location of the script
echo "Validating the script location ..."
if [ ! -a $SCRIPT_LOCATION/install_olap.sh ]
then
	echo \*\*\*
	echo \*\*\* Error: Invalid script location, $SCRIPT_LOCATION
	echo \*\*\* Cannot find the install_olap.sh within specified script location.
	echo \*\*\* Please ensure the script location is correct, and try again.
	echo \*\*\*
	exit 1
fi

#Check the Host, Port, SID
echo "Validating Host, Port, SID"
TNSNAME_CHECK="(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=$HOSTNAME)(PORT=$PORT)))(CONNECT_DATA=(SID=$SID)))"

echo $ORACLE_HOME/bin/sqlplus -S system/$SYSTEM_PASSWORD@$TNSNAME_CHECK @logon.sql
$ORACLE_HOME/bin/sqlplus -S system/$SYSTEM_PASSWORD@"$TNSNAME_CHECK" @logon.sql
if [ $? -ne 0 ]
then
	echo \*\*\*
	echo \*\*\* Error: Cannot logon using the Host, Port, SID supplied.
	echo \*\*\* Please ensure the supplied Host, Port and ORACLE_SID environment
	echo \*\*\* variable are correctly set, and try again.
	echo \*\*\*
	exit 1
fi

if [ $OLAP_INSTALLED != "TRUE" ] 
then
	echo \*\*\*
	echo \*\*\* WARNING: OLAP not setup, some or all of the following
	echo \*\*\* OLAP components are missing from the target database:
	echo \*\*\* OLAP Anaytic Workspace,
	echo \*\*\* Oracle OLAP API,
	echo \*\*\* Oracle Catalog.
	echo \*\*\*
	exit 1
fi

if [ $OLAP_INSTALLED = "TRUE" ]
then
echo "Create Disco catalog ..."
cd $SCRIPT_LOCATION/olap
./mtolap.sh $HOSTNAME $PORT $SID $SYSTEM_PASSWORD $BI_HOME $D4OSYS_PASSWORD $SCRIPT_LOCATION/olap $TNSNAMES
if [ $? -ne 0 ]
then
	echo \*\*\*
	echo \*\*\* Error: mtolap.sh returned with error
	echo \*\*\*
	exit 1
fi
fi

cd $SCRIPT_LOCATION
