#!/bin/ksh

echo \*\*\*
echo \*\*\*
echo \*\*\* This script makes the following assumptions:
echo \*\*\* 1. The script is being executed on the Business Intelligence middle tier server.
echo \*\*\* 2. ORACLE_HOME environment variable is set to the BI Home: Not DB Home
echo \*\*\* 3. The bi_user has not been previously installed.
echo \*\*\* 4. Oracle Command Line for Discoverer has been installed.
echo \*\*\*


#Check for the number of arguments.
if [ $# -ne 5 ]
then	
	echo "Usage: $0 tnsnames_entry system_password default_tablespace temporary_tablespace location_script"
	echo   "tnsnames_entry       The tnsnames entry for the database server."
	echo   "system_password      The SYSTEM password."
	echo   "default_tablespace   The default tablespace for bi_user, in UPPER CASE."
	echo   "temporary_tablespace The temporary tablespace for bi_user, in UPPER CASE."
	echo   "location_script      Fully qualified location of the script."
	echo   ""
	echo   "For example:"
	echo   "$0 tnsentry manager USERS TEMP /tmp/scripts/samples/discoverer"
	exit 1
fi

#Set Common Vars
TNSNAMES=$1
SYSTEM_PASSWORD=$2
DEFAULT_TABLESPACE=$3
TEMPORARY_TABLESPACE=$4
SCRIPT_LOCATION=$5
BI_HOME=$ORACLE_HOME

# Display the arguments to the user to verify they are correct 
echo "You have specified the following arguments to install_rel.sh"

echo "tnsnames_entry	$TNSNAMES"
echo "system_password 	$SYSTEM_PASSWORD"
echo "default_tablespace	$DEFAULT_TABLESPACE"
echo "temporary_tablespace	$TEMPORARY_TABLESPACE"
echo "location_script		$SCRIPT_LOCATION"
echo 

#Check ORACLE_HOME has been set

echo Validating ORACLE_HOME is set.

txt=`env | grep "^ORACLE_HOME="`
if [ "$txt" = "" ]
then
	echo "Env variable ORACLE_HOME not set"
	echo "The ORACLE_HOME must be set to the BI Home and not the DB Home."
	exit 1
fi

#Check bi_user does not exist
echo Validating bi_user does not exist ...
$ORACLE_HOME/bin/sqlplus -S system/$SYSTEM_PASSWORD@$TNSNAMES @user_exists.sql BI_USER
if [ $? -ne 0 ]
then
#Check the SYSTEM password ...
echo Validating SYSTEM password ...
$ORACLE_HOME/bin/sqlplus -S system/$SYSTEM_PASSWORD@$TNSNAMES @logon.sql
if [ $? -ne 0 ]
then
        echo \*\*\*
        echo \*\*\* Error: Cannot authenticate using supplied SYSTEM credentials.
        echo \*\*\* Please validate the supplied password and tnsnames, and try again.
        echo \*\*\*
        exit 1
fi
else
	echo \*\*\*
	echo \*\*\* Error: bi_user already exists.
	echo \*\*\* Please drop the bi_user if a previous installation failed.
	echo \*\*\*
	exit 1
fi

#Check the location of the BI_HOME
echo "Validating the location of BI_HOME exists ..."
if [ ! -a $BI_HOME/bin ]
then
	echo \*\*\*
	echo \*\*\* Error: Invalid location for BI Home.
	echo \*\*\* Please ensure the location of the BI Home is valid, and try again.
	echo \*\*\*
	exit 1
fi

#Check for EUL Command line
echo "Validating EUL Command Line installed - \$BI_HOME/eulapi ..."
if [ ! -a $BI_HOME/bin/eulapi ]
then
	echo \*\*\*
	echo \*\*\* Error: Cannot find EUL Command line - $BI_HOME/bin/eulapi.
	echo \*\*\* Please ensure EUL Command line is installed, and try again.
	echo \*\*\*
	exit 1
fi

#Check tablespaces exist
echo "Validating tablespaces ..."
$ORACLE_HOME/bin/sqlplus -S system/$SYSTEM_PASSWORD@$TNSNAMES @tablespace.sql $DEFAULT_TABLESPACE
if [ $? -ne 0 ]
then
	echo \*\*\*
	echo \*\*\* Error: The tablespace $DEFAULT_TABLESPACE does not exist in the database.
	echo \*\*\* Please ensure you supply a valid default tablespace, and try again.
	echo \*\*\*
	exit 1
fi

$ORACLE_HOME/bin/sqlplus -S system/$SYSTEM_PASSWORD@$TNSNAMES @tablespace.sql $TEMPORARY_TABLESPACE
if [ $? -ne 0 ]
then
	echo \*\*\*
	echo \*\*\* Error: The tablespace $TEMPORARY_TABLESPACE does not exist in the database.
	echo \*\*\* Please ensure you supply a valid temporary tablespace, and try again.
	echo \*\*\*
	exit 1
fi

#Check the location of the script
echo "Validating the script location ..."
if [ ! -a $SCRIPT_LOCATION/install_rel.sh ]
then
	echo \*\*\*
	echo \*\*\* Error: Invalid script location, $SCRIPT_LOCATION
	echo \*\*\* Cannot find the install_rel.sh within specified script location.
	echo \*\*\* Please ensure the script location is correct, and try again.
	echo \*\*\*
	exit 1
fi

#Invoke EUL command line and create bi_user
echo "Creating bi_user through EUL Command Line."
cd $SCRIPT_LOCATION/disco
./mtdisco.sh $TNSNAMES $SYSTEM_PASSWORD $BI_HOME $DEFAULT_TABLESPACE $TEMPORARY_TABLESPACE $SCRIPT_LOCATION/disco
if [ $? -ne 0 ]
then
	echo \*\*\*
	echo \*\*\* Error: mtdisco.sh returned with error
	echo \*\*\*
	exit 1
fi

cd $SCRIPT_LOCATION
